@@ -47,7 +47,7 @@ try:
 except ImportError:
     ansible_version = 'unknown'
 
-COLLECTION_VERSION = "21.6.0"
+COLLECTION_VERSION = "21.8.1"
 CLIENT_APP_VERSION = "%s/" + COLLECTION_VERSION
 IMPORT_EXCEPTION = None
 
@@ -98,6 +98,7 @@ ERROR_MSG = dict(
 )
 
 LOG = logging.getLogger(__name__)
+LOG_FILE = '/tmp/ontap_apis.log'
 
 try:
     from solidfire.factory import ElementFactory
@@ -135,6 +136,16 @@ def na_ontap_host_argument_spec():
     )
 
 
+def na_ontap_host_argument_spec_peer():
+    spec = na_ontap_host_argument_spec()
+    spec.pop('feature_flags')
+    # get rid of default values, as we'll use source values
+    for value in spec.values():
+        if 'default' in value:
+            value.pop('default')
+    return spec
+
+
 def has_feature(module, feature_name):
     feature = get_feature(module, feature_name)
     if isinstance(feature, bool):
@@ -147,6 +158,8 @@ def get_feature(module, feature_name):
         otherwise, use our default
     '''
     default_flags = dict(
+        strict_json_check=True,                 # if true, fail if response.content in not empty and is not valid json
+        trace_apis=False,                       # if true, append ZAPI and REST requests/responses to /tmp/ontap_zapi.txt
         check_required_params_for_none=True,
         classic_basic_authorization=False,      # use ZAPI wrapper to send Authorization header
         deprecation_warning=True,
@@ -154,7 +167,6 @@ def get_feature(module, feature_name):
         sanitize_code_points=[8],               # unicode values, 8 is backspace
         show_modified=True,
         always_wrap_zapi=True,                  # for better error reporting
-        trace_apis=False,                       # if true, append ZAPI and REST requests/responses to /tmp/ontap_zapi.txt
         flexcache_delete_return_timeout=5       # ONTAP bug if too big?
     )
 
@@ -207,24 +219,53 @@ def set_auth_method(module, username, password, cert_filepath, key_filepath):
     return auth_method
 
 
-def setup_na_ontap_zapi(module, vserver=None, wrap_zapi=False):
-    hostname = module.params['hostname']
-    username = module.params['username']
-    password = module.params['password']
-    https = module.params['https']
-    validate_certs = module.params['validate_certs']
-    port = module.params['http_port']
-    version = module.params['ontapi']
-    cert_filepath = module.params['cert_filepath']
-    key_filepath = module.params['key_filepath']
-    auth_method = set_auth_method(module, username, password, cert_filepath, key_filepath)
+def setup_host_options_from_module_params(host_options, module, keys):
+    '''if an option is not set, use primary value.
+       but don't mix up basic and certificate authentication methods
+
+       host_options is updated in place
+       option values are read from module.params
+       keys is a list of keys that need to be added/updated/left alone in host_options
+    '''
+    password_keys = ['username', 'password']
+    certificate_keys = ['cert_filepath', 'key_filepath']
+    use_password = any(host_options.get(x) is not None for x in password_keys)
+    use_certificate = any(host_options.get(x) is not None for x in certificate_keys)
+    if use_password and use_certificate:
+        module.fail_json(
+            msg='Error: host cannot have both basic authentication (username/password) and certificate authentication (cert/key files).')
+    if use_password:
+        exclude_keys = certificate_keys
+    elif use_certificate:
+        exclude_keys = password_keys
+    else:
+        exclude_keys = list()
+    for key in keys:
+        if host_options.get(key) is None and key not in exclude_keys:
+            # use same value as source if no value is given for dest
+            host_options[key] = module.params[key]
+
+
+def setup_na_ontap_zapi(module, vserver=None, wrap_zapi=False, host_options=None):
+    if host_options is None:
+        host_options = module.params
+    hostname = host_options.get('hostname')
+    username = host_options.get('username')
+    password = host_options.get('password')
+    cert_filepath = host_options.get('cert_filepath')
+    key_filepath = host_options.get('key_filepath')
+    https = host_options.get('https')
+    validate_certs = host_options.get('validate_certs')
+    port = host_options.get('http_port')
+    version = host_options.get('ontapi')
     if has_feature(module, 'trace_apis'):
-        logging.basicConfig(filename='/tmp/ontap_apis.log', level=logging.DEBUG)
+        logging.basicConfig(filename=LOG_FILE, level=logging.DEBUG)
         trace = True
     else:
         trace = False
     if has_feature(module, 'always_wrap_zapi'):
         wrap_zapi = True
+    auth_method = set_auth_method(module, username, password, cert_filepath, key_filepath)
 
     if HAS_NETAPP_LIB:
         # set up zapi
@@ -543,7 +584,7 @@ class OntapRestAPI(object):
         self.auth_method = set_auth_method(self.module, self.username, self.password, self.cert_filepath, self.key_filepath)
         self.check_required_library()
         if has_feature(module, 'trace_apis'):
-            logging.basicConfig(filename='/tmp/ontap_apis.log', level=logging.DEBUG, format='%(asctime)s %(levelname)-8s %(message)s')
+            logging.basicConfig(filename=LOG_FILE, level=logging.DEBUG, format='%(asctime)s %(levelname)-8s %(message)s')
 
     def requires_ontap_9_6(self, module_name):
         self.requires_ontap_version(module_name)
@@ -555,8 +596,8 @@ class OntapRestAPI(object):
     def options_require_ontap_version(self, options, version='9.6', use_rest=None):
         current_version = self.get_ontap_version()
         suffix = " - %s" % self.is_rest_error if self.is_rest_error is not None else ""
-        if current_version != (-1, -1):
-            suffix += " - ONTAP version: %s.%s" % current_version
+        if current_version != (-1, -1, -1):
+            suffix += " - ONTAP version: %s.%s.%s" % current_version
         if use_rest is not None:
             suffix += " - using %s" % ('REST' if use_rest else 'ZAPI')
         if isinstance(options, list):
@@ -570,8 +611,25 @@ class OntapRestAPI(object):
             tag = str(options)
         return 'using %s requires ONTAP %s or later and REST must be enabled%s.' % (tag, version, suffix)
 
-    def meets_rest_minimum_version(self, use_rest, minimum_generation, minimum_major):
-        return use_rest and self.get_ontap_version() >= (minimum_generation, minimum_major)
+    def meets_rest_minimum_version(self, use_rest, minimum_generation, minimum_major, minimum_minor=0):
+        return use_rest and self.get_ontap_version() >= (minimum_generation, minimum_major, minimum_minor)
+
+    def fail_if_not_rest_minimum_version(self, module_name, minimum_generation, minimum_major, minimum_minor=0):
+        status_code = self.get_ontap_version_using_rest()
+        msgs = list()
+        if self.use_rest == 'never':
+            msgs.append('Error: REST is required for this module, found: "use_rest: %s"' % self.module.params.get('use_rest'))
+        if self.is_rest_error:
+            msgs.append('Error using REST for version, error: %s.' % self.is_rest_error)
+        if status_code != 200:
+            msgs.append('Error using REST for version, status_code: %s.' % status_code)
+        if msgs:
+            self.module.fail_json(msg='  '.join(msgs))
+        version = self.get_ontap_version()
+        if version < (minimum_generation, minimum_major, minimum_minor):
+            msg = 'Error: ' + self.requires_ontap_version(module_name, '%d.%d' % (minimum_generation, minimum_major))
+            msg += '  Found: %s.%s.%s.' % version
+            self.module.fail_json(msg=msg)
 
     def check_required_library(self):
         if not HAS_REQUESTS:
@@ -601,11 +659,22 @@ class OntapRestAPI(object):
         error_details = None
         headers = self.build_headers(accept, vserver_name, vserver_uuid)
 
+        def check_contents(response):
+            '''json() may fail on an empty value, but it's OK if no response is expected.
+               To avoid false positives, only report an issue when we expect to read a value.
+               The first get will see it.
+            '''
+            if method == 'GET' and has_feature(self.module, 'strict_json_check'):
+                contents = response.content
+                if len(contents) > 0:
+                    raise ValueError("Expecting json, got: %s" % contents)
+
         def get_json(response):
             ''' extract json, and error message if present '''
             try:
                 json = response.json()
             except ValueError:
+                check_contents(response)
                 return None, None
             error = json.get('error')
             return json, error
@@ -760,8 +829,8 @@ class OntapRestAPI(object):
 
     def get_ontap_version(self):
         if self.ontap_version['valid']:
-            return self.ontap_version['generation'], self.ontap_version['major']
-        return -1, -1
+            return self.ontap_version['generation'], self.ontap_version['major'], self.ontap_version['minor']
+        return -1, -1, -1
 
     def get_ontap_version_using_rest(self):
         # using GET rather than HEAD because the error messages are different,
@@ -794,10 +863,10 @@ class OntapRestAPI(object):
         # we're now using 'auto'
         if used_unsupported_rest_properties:
             # force ZAPI if some parameter requires it
-            if self.get_ontap_version() > (9, 5):
+            if self.get_ontap_version()[0:2] > (9, 5):
                 self.module.warn('Falling back to ZAPI because of unsupported option(s) or option value(s) in REST: %s' % used_unsupported_rest_properties)
             return False, None
-        if self.get_ontap_version() in ((9, 4), (9, 5)):
+        if self.get_ontap_version()[0:2] in ((9, 4), (9, 5)):
             # we can't trust REST support on 9.5, and not at all on 9.4
             return False, None
         if status_code == 200:
