@@ -15,8 +15,6 @@ import json
 
 from itertools import chain
 
-from ansible_collections.ansible.netcommon.plugins.module_utils.compat import ipaddress
-
 from ansible.module_utils.common.text.converters import to_text
 
 from ansible.module_utils._text import to_native
@@ -53,6 +51,7 @@ API_APPS_ENDPOINTS = dict(
         "interfaces",
         "interface_templates",
         "inventory_items",
+        "locations",
         "manufacturers",
         "platforms",
         "power_feeds",
@@ -68,6 +67,7 @@ API_APPS_ENDPOINTS = dict(
         "rear_port_templates",
         "regions",
         "sites",
+        "site_groups",
         "virtual_chassis",
     ],
     extras=["tags"],
@@ -103,12 +103,15 @@ QUERY_TYPES = dict(
     group="slug",
     installed_device="name",
     import_targets="name",
+    location="slug",
     manufacturer="slug",
     nat_inside="address",
     nat_outside="address",
     parent_region="slug",
+    parent_tenant_group="slug",
     power_panel="name",
     power_port="name",
+    power_port_template="name",
     platform="slug",
     prefix_role="slug",
     primary_ip="address",
@@ -125,9 +128,11 @@ QUERY_TYPES = dict(
     route_targets="name",
     slug="slug",
     site="slug",
+    site_group="slug",
     tenant="slug",
     tenant_group="slug",
     time_zone="timezone",
+    virtual_chassis="name",
     virtual_machine="name",
     virtual_machine_role="slug",
     vlan="name",
@@ -165,6 +170,7 @@ CONVERT_TO_ID = {
     "interface_template": "interface_templates",
     "ip_addresses": "ip_addresses",
     "ipaddresses": "ip_addresses",
+    "location": "locations",
     "lag": "interfaces",
     "manufacturer": "manufacturers",
     "master": "devices",
@@ -172,8 +178,10 @@ CONVERT_TO_ID = {
     "nat_outside": "ip_addresses",
     "platform": "platforms",
     "parent_region": "regions",
+    "parent_tenant_group": "tenant_groups",
     "power_panel": "power_panels",
     "power_port": "power_ports",
+    "power_port_template": "power_port_templates",
     "prefix_role": "roles",
     "primary_ip": "ip_addresses",
     "primary_ip4": "ip_addresses",
@@ -187,8 +195,11 @@ CONVERT_TO_ID = {
     "rear_port_template": "rear_port_templates",
     "rir": "rirs",
     "route_targets": "route_targets",
+    # Just a placeholder as scope can be several different types including sites.
+    "scope": "sites",
     "services": "services",
     "site": "sites",
+    "site_group": "site_groups",
     "tags": "tags",
     "tagged_vlans": "vlans",
     "tenant": "tenants",
@@ -229,6 +240,7 @@ ENDPOINT_NAME_MAPPING = {
     "interface_templates": "interface_template",
     "inventory_items": "inventory_item",
     "ip_addresses": "ip_address",
+    "locations": "location",
     "manufacturers": "manufacturer",
     "platforms": "platform",
     "power_feeds": "power_feed",
@@ -250,6 +262,7 @@ ENDPOINT_NAME_MAPPING = {
     "route_targets": "route_target",
     "services": "services",
     "sites": "site",
+    "site_groups": "site_group",
     "tags": "tags",
     "tenants": "tenant",
     "tenant_groups": "tenant_group",
@@ -293,14 +306,16 @@ ALLOWED_QUERY_PARAMS = {
     "interface": set(["name", "device", "virtual_machine"]),
     "interface_template": set(["name", "device_type"]),
     "inventory_item": set(["name", "device"]),
-    "ip_address": set(["address", "vrf", "device", "interface"]),
-    "ip_addresses": set(["address", "vrf", "device", "interface"]),
-    "ipaddresses": set(["address", "vrf", "device", "interface"]),
+    "ip_address": set(["address", "vrf", "device", "interface", "assigned_object"]),
+    "ip_addresses": set(["address", "vrf", "device", "interface", "assigned_object"]),
+    "ipaddresses": set(["address", "vrf", "device", "interface", "assigned_object"]),
     "lag": set(["name"]),
+    "location": set(["slug"]),
     "manufacturer": set(["slug"]),
     "master": set(["name"]),
     "nat_inside": set(["vrf", "address"]),
     "parent_region": set(["slug"]),
+    "parent_tenant_group": set(["slug"]),
     "platform": set(["slug"]),
     "power_feed": set(["name", "power_panel"]),
     "power_outlet": set(["name", "device"]),
@@ -330,10 +345,10 @@ ALLOWED_QUERY_PARAMS = {
     "termination_a": set(["name", "device", "virtual_machine"]),
     "termination_b": set(["name", "device", "virtual_machine"]),
     "untagged_vlan": set(["group", "name", "site", "vid", "vlan_group", "tenant"]),
-    "virtual_chassis": set(["master"]),
+    "virtual_chassis": set(["name", "master"]),
     "virtual_machine": set(["name", "cluster"]),
     "vlan": set(["group", "name", "site", "tenant", "vid", "vlan_group"]),
-    "vlan_group": set(["slug", "site"]),
+    "vlan_group": set(["slug", "site", "scope"]),
     "vrf": set(["name", "tenant"]),
 }
 
@@ -347,9 +362,11 @@ QUERY_PARAMS_IDS = set(
         "rir",
         "vrf",
         "site",
+        "scope",
         "tenant",
         "type",
         "virtual_machine",
+        "rack",
     ]
 )
 
@@ -385,10 +402,13 @@ REQUIRED_ID_FIND = {
 # This is used to map non-clashing keys to Netbox API compliant keys to prevent bad logic in code for similar keys but different modules
 CONVERT_KEYS = {
     "assigned_object": "assigned_object_id",
+    "scope": "scope_id",
     "circuit_type": "type",
     "cluster_type": "type",
     "cluster_group": "group",
     "parent_region": "parent",
+    "parent_tenant_group": "parent",
+    "power_port_template": "power_port",
     "prefix_role": "role",
     "rack_group": "group",
     "rack_role": "role",
@@ -410,12 +430,14 @@ SLUG_REQUIRED = {
     "device_roles",
     "device_types",
     "ipam_roles",
+    "locations",
     "rack_groups",
     "rack_roles",
     "regions",
     "rirs",
     "roles",
     "sites",
+    "site_groups",
     "tags",
     "tenants",
     "tenant_groups",
@@ -425,6 +447,16 @@ SLUG_REQUIRED = {
     "vlan_groups",
 }
 
+SCOPE_TO_ENDPOINT = {
+    "dcim.location": "locations",
+    "dcim.rack": "racks",
+    "dcim.region": "regions",
+    "dcim.site": "sites",
+    "dcim.sitegroup": "site_groups",
+    "virtualization.cluster": "clusters",
+    "virtualization.clustergroup": "cluster_groups",
+}
+
 NETBOX_ARG_SPEC = dict(
     netbox_url=dict(type="str", required=True),
     netbox_token=dict(type="str", required=True, no_log=True),
@@ -602,8 +634,8 @@ class NetboxModule(object):
             if self.endpoint == "power_panels" and key == "rack_group":
                 temp_dict[key] = data[key]
             elif key in CONVERT_KEYS:
-                # This will keep the original key for assigned_object, but also convert to assigned_object_id
-                if key == "assigned_object":
+                # This will keep the original key for keys in list, but also convert it.
+                if key in ("assigned_object", "scope"):
                     temp_dict[key] = data[key]
                 new_key = CONVERT_KEYS[key]
                 temp_dict[new_key] = data[key]
@@ -665,6 +697,8 @@ class NetboxModule(object):
             parent = module_data["termination_a_type"]
         elif parent == "termination_b" and module_data.get("termination_b_type"):
             parent = module_data["termination_b_type"]
+        elif parent == "scope":
+            parent = ENDPOINT_NAME_MAPPING[SCOPE_TO_ENDPOINT[module_data["scope_type"]]]
 
         query_dict = dict()
         if user_query_params:
@@ -749,6 +783,25 @@ class NetboxModule(object):
                 }
                 query_dict.update(rear_port_template)
 
+        elif parent == "power_port" and self.endpoint == "power_outlets":
+            if isinstance(module_data.get("power_port"), str):
+                power_port = {
+                    "device_id": module_data.get("device"),
+                    "name": module_data.get("power_port"),
+                }
+                query_dict.update(power_port)
+
+        elif (
+            parent == "power_port_template"
+            and self.endpoint == "power_outlet_templates"
+        ):
+            if isinstance(module_data.get("power_port_template"), str):
+                power_port_template = {
+                    "devicetype_id": module_data.get("device_type"),
+                    "name": module_data.get("power_port_template"),
+                }
+                query_dict.update(power_port_template)
+
         elif "_template" in parent:
             if query_dict.get("device_type"):
                 query_dict["devicetype_id"] = query_dict.pop("device_type")
@@ -836,6 +889,9 @@ class NetboxModule(object):
                     endpoint = CONVERT_TO_ID[data.get("termination_b_type")]
                 elif k == "assigned_object":
                     endpoint = "interfaces"
+                elif k == "scope":
+                    # Determine endpoint name for scope ID resolution
+                    endpoint = SCOPE_TO_ENDPOINT[data["scope_type"]]
                 else:
                     endpoint = CONVERT_TO_ID[k]
                 search = v
@@ -875,10 +931,22 @@ class NetboxModule(object):
                         else:
                             self._handle_errors(msg="%s not found" % (list_item))
                 else:
-                    if k in ["lag", "rear_port", "rear_port_template"]:
+                    if k in [
+                        "lag",
+                        "rear_port",
+                        "rear_port_template",
+                        "power_port",
+                        "power_port_template",
+                    ]:
                         query_params = self._build_query_params(
                             k, data, user_query_params
                         )
+                    elif k == "scope":
+                        query_params = {
+                            QUERY_TYPES.get(
+                                ENDPOINT_NAME_MAPPING[endpoint], "q"
+                            ): search
+                        }
                     else:
                         query_params = {QUERY_TYPES.get(k, "q"): search}
                     query_id = self._nb_endpoint_get(nb_endpoint, query_params, k)
@@ -928,7 +996,13 @@ class NetboxModule(object):
                         if sub_data_type == "slug":
                             data[k][subk] = self._to_slug(subv)
             else:
-                data_type = QUERY_TYPES.get(k, "q")
+                if k == "scope":
+                    data_type = QUERY_TYPES.get(
+                        ENDPOINT_NAME_MAPPING[SCOPE_TO_ENDPOINT[data["scope_type"]]],
+                        "q",
+                    )
+                else:
+                    data_type = QUERY_TYPES.get(k, "q")
                 if data_type == "slug":
                     data[k] = self._to_slug(v)
                 elif data_type == "timezone":
